// BeAIM - BuddyList.cpp
// The Buddy List module

#include <Application.h>
#include <View.h>
#include <TabView.h>
#include <MenuBar.h>
#include <MenuItem.h>	
#include <Alert.h>
#include <ScrollView.h>
#include <SupportKit.h>
#include <stdlib.h>
#include <stdio.h>
#include "BuddyList.h"
#include "MiscStuff.h"
#include "PeopleEdit.h"
#include "Say.h"
#include "Globals.h"

//=========================================================================

const int LOGO_HEIGHT = 40;

//=========================================================================

BuddyList::BuddyList(BRect frame)
	     : BWindow(frame, "Buddy List", B_TITLED_WINDOW, B_NOT_ZOOMABLE, B_ALL_WORKSPACES )
{
	BRect r; 
	BTabView *tabView; 
	BBitmap* smallLogo = NULL;
	PeopleTab *onlinetab, *setuptab;
	BMenuItem* menuItem;
	r = Bounds();
	
	// Add the menu bar stuff
	BMenu *beMenu, *statMenu;
	menubar = new BMenuBar(r, "menu_bar");
	beMenu = new BMenu("BeAIM");
	beMenu->AddItem(new BMenuItem("Contact User" B_UTF8_ELLIPSIS, new BMessage(BEAIM_SEND_NEW_MESSAGE) ));
	beMenu->AddItem(new BMenuItem("Get User Info" B_UTF8_ELLIPSIS, new BMessage(BEAIM_GET_PERSON_INFO) ));
	beMenu->AddItem(new BMenuItem("Search by email address" B_UTF8_ELLIPSIS, new BMessage(BEAIM_SEARCH_BY_EMAIL) ));
	beMenu->AddItem(new BMenuItem("Preferences" B_UTF8_ELLIPSIS, new BMessage(BEAIM_OPEN_PREFS) ));
	beMenu->AddSeparatorItem();
	beMenu->AddItem(menuItem=new BMenuItem("About BeAIM" B_UTF8_ELLIPSIS, new BMessage(B_ABOUT_REQUESTED) ));
	menuItem->SetTarget(APP_SIGNATURE);
	beMenu->AddSeparatorItem();
	beMenu->AddItem(menuItem=new BMenuItem("Logout" B_UTF8_ELLIPSIS, new BMessage(BEAIM_LOGOUT) ));
	menuItem->SetEnabled(false);
	beMenu->AddItem(new BMenuItem("Quit", new BMessage(B_QUIT_REQUESTED) ));
	//beMenu->AddItem(new BMenuItem("Setup Away messages" B_UTF8_ELLIPSIS, NULL ));
	
	statMenu = new BMenu( "Status" );
	statMenu->SetEnabled( false );
	BMenuItem* availItem = new BMenuItem("Available", NULL, '1');
	availItem->SetMarked( true );
	statMenu->AddItem( availItem );
	statMenu->AddItem(new BMenuItem("Away", NULL, '2'));
	statMenu->AddItem(new BMenuItem("Gone fishin'", NULL, '3'));
	statMenu->SetRadioMode( true );

	menubar->AddItem(beMenu);
	menubar->AddItem(statMenu);
	AddChild(menubar);	

	// Add the logo thinger
	r.top = menubar->Bounds().bottom + 1.0;
	r.bottom = r.top + LOGO_HEIGHT;
	GetBitmapFromResources( smallLogo, 942 );
	logoView = new BitmapView( r, smallLogo );
	AddChild( logoView );
	
	// add the (currently empty) infoView
	r.top = r.bottom - 1;
	iView = new BuddyListInfoView( r );
	AddChild( iView );

	// Add the tabview control
	r = Bounds();
	r.top = menubar->Bounds().bottom +logoView->Bounds().bottom + 1.0;
	tabView = new BTabView(r, "tab_view"); 
	tabView->SetViewColor(216,216,216,0);
	AddChild( tabView );	
	tabView->SetTabWidth( B_WIDTH_FROM_WIDEST );  		
	
	// Add the tabs themselves
	r = tabView->Bounds();
	r.InsetBy(5,5); 
	r.bottom -= tabView->TabHeight(); 
	onlinetab = new PeopleTab(0, this);
	tabView->AddTab(onlineview = new BuddyListView(r), onlinetab);
	onlinetab->SetLabel("Online");
	onlinetab->SetControlView( onlineview );
	setuptab = new PeopleTab(1, this);
	tabView->AddTab(setupview = new BuddyListSetupView(r), setuptab);
	setuptab->SetLabel("List Setup");
	setuptab->SetControlView( setupview );
	tabView->Select( 0 );
	
	// set up 1-second pulses, to handle the buddy list change updates
	SetPulseRate( 1000000 );
	
	// add shortcuts to handle prev and next commands from the buddylist
	AddShortcut( ',', 0, new BMessage(BEAIM_PREV_CHAT_WINDOW) );
	AddShortcut( '.', 0, new BMessage(BEAIM_NEXT_CHAT_WINDOW) );
	
	// add the "secret" SendData shortcut
	AddShortcut( '\\', B_SHIFT_KEY, new BMessage(BEAIM_SEND_ARBITRARY_DATA) );
	
	// load the prefs
	iView->doIdleTime = false;
	LoadPrefs();
}

//-------------------------------------------------------------------------

BuddyList::~BuddyList()
{
}

//-------------------------------------------------------------------------

bool BuddyList::QuitRequested()
{
	be_app->PostMessage(B_QUIT_REQUESTED);
	return(true);
}

//-------------------------------------------------------------------------

void BuddyList::MessageReceived( BMessage* message ) {

	// Some variables needed for various messages
	BMessenger messenger(APP_SIGNATURE);
	BOutlineListView* source = NULL;
	BListItem* currentItem = NULL;
	PeopleItem* currentBuddyItem = NULL;	
	char ScreenName[DISPLAY_NAME_MAX];
	BMessage* sendMessage;
	int32 Selected, status;

	switch(message->what) {

		// The user has double-clicked on a buddy. Send a message to open up an IM window.
		case BEAIM_BUDDY_INVOKED:
			message->FindPointer( "source", (void**)&source );
			Selected = source->CurrentSelection();
			currentItem = source->ItemAt( Selected );
			if( (currentBuddyItem = dynamic_cast<PeopleItem*>(currentItem)) == 0 )
				return;
			strcpy( ScreenName, currentBuddyItem->GetPersonName() );
			sendMessage = new BMessage( BEAIM_OPEN_USER_WINDOW );
			sendMessage->AddInt32( "wtype", (int32)USER_MESSAGE_TYPE );
			sendMessage->AddInt32( "proto", (int32)currentBuddyItem->GetProto() );			
			sendMessage->AddString( "userid", ScreenName );
			messenger.SendMessage( sendMessage );
			break;

		// This whole thing is kind of hackish. I want to redo it later.
		case BEAIM_ONCOMING_BUDDY:
			if( unsigned(message->FindInt32("idletime")) > 10 )
				status = BS_IDLE;
			else
				status = BS_ACTIVE;

			// Set the status... if this is a change it won't fail and we won't need to bother adding				
			list->SetPersonStatus( (unsigned)message->FindInt32("proto"), 
								   (char*)message->FindString("userid"), status );
			break;
			
		case BEAIM_OFFGOING_BUDDY:
			list->SetPersonStatus( (unsigned)message->FindInt32("proto"), 
								   (char*)message->FindString("userid"), BS_OFFLINE );
			break;
			
		case BEAIM_EDIT_BUDDY:
			setupview->EditItem();
			break;

		case BEAIM_OPEN_PREFS:
			messenger.SendMessage( message );
			break;
			
		case BEAIM_RELOAD_PREF_SETTINGS:
			LoadPrefs();
			break;
			
		case BEAIM_DELETE_PERSONORGROUP:
			setupview->RemoveItem();
			break;
			
		case BEAIM_TRY_ADD_PERSON:
			setupview->AddItem( ET_ADDPERSON );
			break;
			
		case BEAIM_TRY_ADD_GROUP:
			setupview->AddItem( ET_ADDGROUP );
			break;						
			
		case BEAIM_ADD_PERSON:
			list->AddPerson( (unsigned int)message->FindInt32("newproto"),
							 (char*)message->FindString( "newname"),
							 (char*)message->FindString("group"),
							 (bool)message->FindBool("commitnow") );
			break;
		
		case BEAIM_ADD_GROUP:
			list->AddGroup( (char*)message->FindString( "newname" ) );
			break;		
		
		case BEAIM_CHANGE_PERSON:
			list->ChangePerson( (unsigned)message->FindInt32("oldproto"),
								(char*)message->FindString("oldname"),
								(unsigned)message->FindInt32("newproto"),
								(char*)message->FindString("newname"),
								(char*)message->FindString("group") );
			break;
		
		case BEAIM_CHANGE_GROUP:
			list->ChangeGroup( (char*)message->FindString("oldname"),
							   (char*)message->FindString("newname") );
			break;

		case BEAIM_SEND_ARBITRARY_DATA:
		case BEAIM_PREV_CHAT_WINDOW:
		case BEAIM_NEXT_CHAT_WINDOW:
			be_app->PostMessage( message );
			break;
			
		case BEAIM_SEND_NEW_MESSAGE:
			sendMessage = new BMessage(BEAIM_OPEN_USER_WINDOW);
			sendMessage->AddInt32( "wtype", (int32)USER_MESSAGE_TYPE );
			sendMessage->AddInt32( "proto", AIM_PROTOCOL );
			windows->OpenInputWindow( "Contact User", "Screen Name:", sendMessage, this, 
									  NULL, DISPLAY_NAME_MAX, true, "userid" );
			break;
			
		case BEAIM_SEARCH_BY_EMAIL:
			sendMessage = new BMessage(BEAIM_OPEN_EMAIL_SEARCH);
			sendMessage->AddInt32( "proto", AIM_PROTOCOL );
			windows->OpenInputWindow( "Search for User", "email address:", sendMessage, this, 
									  NULL, 100, true, "email" );
			break;
			
		case BEAIM_GET_PERSON_INFO:
			sendMessage = new BMessage(BEAIM_OPEN_USER_WINDOW);
			sendMessage->AddInt32( "proto", AIM_PROTOCOL );
			sendMessage->AddInt32( "wtype", (int32)USER_INFO_TYPE );
			sendMessage->AddPointer( "poswindow", this );
			windows->OpenInputWindow( "Get User Info", "Screen Name:", sendMessage, this, 
									  NULL, DISPLAY_NAME_MAX, true, "userid" );		
			break;
			
		default:
			BWindow::MessageReceived(message);
			break;
	}
}

//-------------------------------------------------------------------------

void BuddyList::LoadPrefs() {

	// load and implement the "all workspaces" setting
	bool allWorkspaces = prefs->ReadBool("bl_aw", true);
	if( allWorkspaces )
		SetWorkspaces( B_ALL_WORKSPACES );
	else
		SetWorkspaces( B_CURRENT_WORKSPACE );
		
	// load and implement the idle time setting
	bool doIdle = prefs->ReadBool("gm_id", true);
	iView->doIdleTime = doIdle;
	if( doIdle )
		PostAppMessage( new BMessage(BEAIM_NOW_ACTIVE) );
}

//=========================================================================

BuddyListInfoView::BuddyListInfoView(BRect rect)
	   	   : BView(rect, "BuddyList", B_FOLLOW_LEFT_RIGHT, B_WILL_DRAW | B_PULSE_NEEDED )
{
	SetFont(be_bold_font);
	SetFontSize(24);
	SetViewColor( 216, 216, 216 );
	
	// reset the idle counter
	idleCount = 0;
}

//-------------------------------------------------------------------------

// update the change list... pulse is used on this particular view because it is visible
//   whenever the buddylist is visible... pulse stops occasionally for the others
void BuddyListInfoView::Pulse() {

	// make any changes that need to be done in the list
	list->UpdateChangeList();
	
	// do the idle time stuff
	if( doIdleTime )
		DoIdleStuff();
}

//-------------------------------------------------------------------------

void BuddyListInfoView::DoIdleStuff() {

	bigtime_t idle;
	
	// get the idle time
	idle = idle_time();
	
	// bigger than 10 minutes? (600 seconds)
	if( idle > 600000000 ) {

		// did we just now get idle? If so, send an idle message to AIM
		if( !idleCount ) {
			BMessage* idleMsg = new BMessage(BEAIM_NOW_IDLE);
			PostAppMessage(idleMsg);
			printf( "Just idled!\n" );
		}

		// every 10 minutes, fire off an idle event to keep AIM happy (required?)
		if( ++idleCount >= 601 ) {

			// go ahead, fire one off
			BMessage* pulseMsg = new BMessage(BEAIM_SEND_IDLE_PULSE);
			PostAppMessage(pulseMsg);
			printf( "porko!\n" );
			
			// reset the idle counter to 1, not zero...
			//  zero is reserved for not being idle
			idleCount = 1;
		}
	}
	
	// did we just get "un-idle"?
	else {
	
		// if idleCount != 0, then we were idle before
		if( idleCount ) {

			// send out a non-idle message
			BMessage* activeMsg = new BMessage(BEAIM_NOW_ACTIVE);
			PostAppMessage(activeMsg);

			// set it to be non-idle
			idleCount = 0;
		}
	}		
}


//=========================================================================

BuddyListView::BuddyListView(BRect rect)
	   	   : BView(rect, "BuddyList", B_FOLLOW_ALL, B_WILL_DRAW | B_PULSE_NEEDED )
{
	SetViewColor( 216, 216, 216 );
	BFont baseFont; 
	baseFont.SetSize( 12.0 );
	
	// Add the outline control
	BRect r = Bounds();
	r.InsetBy( 1.0, 1.0 );
	online = new ClickableListView( r, "online_list", B_SINGLE_SELECTION_LIST, B_FOLLOW_ALL );
	online->SetFont( &baseFont );
	online->SetInvocationMessage( new BMessage(BEAIM_BUDDY_INVOKED) );	
	
	// set up the scroller
	scroller = new BScrollView( "scroller2", online, B_FOLLOW_ALL, 0, false, true, B_NO_BORDER );
	//AddChild( online );
	AddChild( scroller );
	
	// tell the list that the display list is there
	list->SetDisplayList( online );
}

//-------------------------------------------------------------------------

void BuddyListView::Draw( BRect )
{
	BRect r = Bounds();
	SetHighColor( 156, 154, 156 );		// dark grey
	StrokeRect( r );
}

//-------------------------------------------------------------------------

void BuddyListView::DoResizeStuff( float width, float height ) {

	ResizeTo( width, height );
	
	// Ugh... resizing the scroller stuff is a pain!
	online->ResizeTo( width - B_V_SCROLL_BAR_WIDTH - 2, height - 2 );
	scroller->ResizeTo( width - 2, height - 2 );
	scroller->ScrollBar(B_VERTICAL)->MoveTo( width - B_V_SCROLL_BAR_WIDTH - 1, -1 );
	scroller->ScrollBar(B_VERTICAL)->ResizeTo( B_V_SCROLL_BAR_WIDTH, height );
	scroller->ScrollBar(B_VERTICAL)->SetValue(0);
}

//=========================================================================

ClickableListView::ClickableListView( BRect frame, char* name, list_view_type type, uint32 flags )
				  					: BOutlineListView( frame, name, type, flags )
{
	userPopup = new BPopUpMenu( "userpopup", false );
	userPopup->AddItem( new BMenuItem( "Contact" B_UTF8_ELLIPSIS, NULL ) );
	userPopup->AddItem( new BMenuItem( "Get Info" B_UTF8_ELLIPSIS, NULL ) );
}

//-------------------------------------------------------------------------

ClickableListView::~ClickableListView() {
	delete userPopup;
}

//-------------------------------------------------------------------------

void ClickableListView::MouseDown( BPoint cursor ) {

	uint32 buttons;
	int32 clickIndex;
	PeopleItem* clickItem;
	BMessage* popupMsg;
	BMenuItem* selected;
	int32 option;

	// get the mouse position
	GetMouse( &cursor, &buttons );
		
	// it was a right mouse button click, popup the menu
	if( buttons & B_SECONDARY_MOUSE_BUTTON ) {
		
		// get the index of the item the user clicked on, and select it
		clickIndex = IndexOf(cursor);
		if( clickIndex < 0 )			// not an item?
			return;
		Select(clickIndex);

		// get a pointer to the item, and check to see if it's a person or not
		clickItem = dynamic_cast<PeopleItem*>( ItemAt(clickIndex) );
		if( !clickItem )
			return;

		// it's all good... display the popup menu
		ConvertToScreen( &cursor );
		selected = userPopup->Go( cursor );

		// if the user selected something, take action based on the choice
		if( selected ) {
			option = userPopup->IndexOf(selected);
			popupMsg = new BMessage(BEAIM_OPEN_USER_WINDOW);
			popupMsg->AddString( "userid", clickItem->GetPersonName() );
			popupMsg->AddInt32( "proto", AIM_PROTOCOL );
			popupMsg->AddPointer( "poswindow", Window() );
			switch( option ) {
				case 0: popupMsg->AddInt32( "wtype", (int32)USER_MESSAGE_TYPE );
					break;
				case 1: popupMsg->AddInt32( "wtype", (int32)USER_INFO_TYPE );
					break;			
			}
			PostAppMessage( popupMsg );			
		}
	} else
		BOutlineListView::MouseDown( cursor );
}

//=========================================================================

BuddyListSetupView::BuddyListSetupView(BRect rect)
	   	   : BView(rect, "BuddyList", B_FOLLOW_ALL, B_WILL_DRAW | B_FULL_UPDATE_ON_RESIZE )
{
	SetFont(be_bold_font);
	SetViewColor( 216, 216, 216 );
	BFont baseFont; 
	baseFont.SetSize( 12.0 );	

	// Make the outline control
	BRect r = Bounds();
	r.InsetBy( 1.0, 1.0 );
	r.bottom -= 26;
	r.right -= B_V_SCROLL_BAR_WIDTH + 2;
	buddies = new BOutlineListView( r, "buddies_setup_list", B_SINGLE_SELECTION_LIST, B_FOLLOW_ALL );
	buddies->SetFont( &baseFont );

	// Make the scrollview control and attach it
	scroller = new BScrollView( "scroller1", buddies, B_FOLLOW_ALL, 0, false, true, B_NO_BORDER );
	buddies->SetInvocationMessage( new BMessage(BEAIM_EDIT_BUDDY) );
	AddChild( scroller );	

	// Add some picture buttons
	BRect btnRect( r.right - 27, r.bottom + 4, r.right-1, r.bottom + 25 );
	BPicture* blank = new BPicture;
	removeBtn = new BPictureButton( btnRect, "remove_person", blank, blank, new BMessage(BEAIM_DELETE_PERSONORGROUP),
									B_ONE_STATE_BUTTON, B_FOLLOW_RIGHT | B_FOLLOW_BOTTOM );
	removeBtn->SetTarget( this );
	btnRect.left = 0;
	btnRect.right = btnRect.left + 26;
	addGroupBtn = new BPictureButton( btnRect, "add_group", blank, blank, new BMessage(BEAIM_TRY_ADD_GROUP),
									B_ONE_STATE_BUTTON, B_FOLLOW_LEFT | B_FOLLOW_BOTTOM );
	addGroupBtn->SetTarget( this );	
	btnRect.left += 30;
	btnRect.right += 30;
	addBuddyBtn = new BPictureButton( btnRect, "add_person", blank, blank, new BMessage(BEAIM_TRY_ADD_PERSON),
									B_ONE_STATE_BUTTON, B_FOLLOW_LEFT | B_FOLLOW_BOTTOM );
	addBuddyBtn->SetTarget( this );	
	AddChild( addGroupBtn );
	AddChild( addBuddyBtn );
	AddChild( removeBtn );
		
	// tell the list that the setup list is there
	list->SetSetupList( buddies );
	
	buttonsDone = false;
	delete blank;
}

//-------------------------------------------------------------------------

// Set up the button glyphs
void BuddyListSetupView::AttachedToWindow() {

	if( !buttonsDone ) {
	
		BPicture* btnPict = NULL; 
		
		MakeButtonPicture( btnPict, this, 101 );
		addBuddyBtn->SetEnabledOn( btnPict );
		
		MakeButtonPicture( btnPict, this, 100 );
		addBuddyBtn->SetEnabledOff( btnPict );
		
		MakeButtonPicture( btnPict, this, 103 );
		addGroupBtn->SetEnabledOn( btnPict );
		
		MakeButtonPicture( btnPict, this, 102 );
		addGroupBtn->SetEnabledOff( btnPict );
		
		MakeButtonPicture( btnPict, this, 105 );
		removeBtn->SetEnabledOn( btnPict );
		
		MakeButtonPicture( btnPict, this, 104 );
		removeBtn->SetEnabledOff( btnPict );				
		
		buttonsDone = true;
		delete btnPict;
	}
}

//-------------------------------------------------------------------------

void BuddyListSetupView::Draw( BRect )
{
	BRect r = Bounds();
	r.bottom -= 26;	
	SetHighColor( 156, 154, 156 );		// dark grey
	StrokeRect( r );
}

//-------------------------------------------------------------------------

void BuddyListSetupView::RemoveItem()
{
	int32 Selected;
	BListItem* currentItem;
	PeopleSetupItem* currentSetupItem;
	BAlert* alert;
	PersonInfo* pi;
	char message[255];
	editType type;

	// Get the currently selected item
	Selected = buddies->CurrentSelection();
	currentItem = buddies->ItemAt( Selected );
	if( (currentSetupItem = dynamic_cast<PeopleSetupItem*>(currentItem)) == NULL )
		return;
		
	// Was this a person item?
	if( currentSetupItem->OutlineLevel() ) {
		pi = list->FindPerson( currentSetupItem->GetProtocol(), currentSetupItem->GetPersonName() );
		if( !pi )
			return;
		sprintf( message, "Are you sure you want to delete\n'%s' from the list?", pi->name );
		type = ET_ADDPERSON;
	}
	
	// No, it was a group.
	else {
		sprintf( message, "Are you sure you want to delete\nthe group '%s'?", currentSetupItem->GetPersonName() );
		type = ET_ADDGROUP;
	}
	
	// ask for permission, then remove the sucker
	alert = new BAlert("title", message, "Cancel", "Delete", NULL, B_WIDTH_AS_USUAL, B_EVEN_SPACING, B_WARNING_ALERT);
	alert->SetShortcut( 0, B_ESCAPE );
	if( alert->Go() ) {
		if( type == ET_ADDPERSON )
			list->RemovePerson( currentSetupItem->GetProtocol(), currentSetupItem->GetPersonName() );
		else
			list->RemoveGroup( currentSetupItem->GetPersonName() );
	}
}

//-------------------------------------------------------------------------

void BuddyListSetupView::AddItem( editType type ) {

	BRect frame( 0, 0, 200, 85 );
	PersonAddEditWindow* dlg;

	// are there any groups?
	if( type == ET_ADDPERSON && !list->CountGroups() ) {
		Say( "You need to add a group first." );
		return;
	}

	// tweak the frame and open the dialog
	windows->MakeDialogFrame( frame, Window() );
 	dlg = new PersonAddEditWindow( frame, Window(), type, NULL );
	dlg->SetNow( false );
 	dlg->Show();
}

//-------------------------------------------------------------------------

void BuddyListSetupView::EditItem()
{
	int32 Selected;
	BRect itemFrame, frame;
	BListItem* currentItem;
	PeopleSetupItem* currentSetupItem;
	BPoint point;
	frame = BRect( 0, 0, 200, 85 );
	editType type;
	PersonAddEditWindow* dlg;
	PersonInfo* pi;

	// Get the currently selected item
	Selected = buddies->CurrentSelection();
	currentItem = buddies->ItemAt( Selected );
	if( (currentSetupItem = dynamic_cast<PeopleSetupItem*>(currentItem)) == NULL )
		return;
		
	// Was this a person item?
	if( currentSetupItem->OutlineLevel() ) {
		pi = list->FindPerson( currentSetupItem->GetProtocol(), currentSetupItem->GetPersonName() );
		if( !pi )
			return;
		type = ET_PERSONEDIT;
	}

	// No, it was a group. Copy in the group name (hackish, but it works for now)
	else {
		pi = new PersonInfo;
		strcpy( pi->name, currentSetupItem->GetPersonName() );
		pi->proto = 0;
		type = ET_GROUPEDIT;
	}

	// Tweak the item frame
	itemFrame = buddies->ItemFrame( Selected );
	point = itemFrame.LeftTop();
	point.y -= scroller->ScrollBar(B_VERTICAL)->Value();
	ConvertToScreen( &point );
	windows->MakeDialogFrame( frame, point );

	// Open the dialog
 	dlg = new PersonAddEditWindow( frame, Window(), type, pi);
 	dlg->SetNow(false);
 	dlg->Show();
}

//-------------------------------------------------------------------------

void BuddyListSetupView::DoResizeStuff( float width, float height ) {

	ResizeTo( width, height );
	addGroupBtn->MoveTo( 0, height - 21 );
	addBuddyBtn->MoveTo( 30, height - 21 );
	removeBtn->MoveTo( width - 26, height - 21 );

	// Ugh... resizing the scroller stuff is a pain!
	buddies->ResizeTo( width - B_V_SCROLL_BAR_WIDTH - 2, height - 28 );
	scroller->ResizeTo( width - 2, height - 28 );
	scroller->ScrollBar(B_VERTICAL)->MoveTo( width - B_V_SCROLL_BAR_WIDTH - 1, -1 );	
	scroller->ScrollBar(B_VERTICAL)->ResizeTo( B_V_SCROLL_BAR_WIDTH, height - 26 );
	scroller->ScrollBar(B_VERTICAL)->SetValue(0);
}

//=========================================================================

// Notes: when you set a tab up with a view that has B_FOLLOW_ALL (or whatever), it only resizes
//   itself when it is the active tab. So this is a major hack that tells the view to resize itself
//   correctly just before it is shown. Sometimes the BeAPI, nice as it is, seems really dumb!

PeopleTab::PeopleTab( int32 num, BuddyList* mn ) {
	view = NULL;
	number = num;
	main = mn;
}

//-------------------------------------------------------------------------

void PeopleTab::SetControlView( BView* vw ) {
	view = vw;
}

//-------------------------------------------------------------------------

void PeopleTab::Select( BView* vw ) {

	BRect rect = main->Bounds();
	rect.bottom -= (51 + LOGO_HEIGHT);
	rect.right -= 12;

	if( view ) {
		if( number == 0 ) {
			BuddyListView* blist = dynamic_cast<BuddyListView*>( view );		
			if( blist )
				blist->DoResizeStuff( rect.Width(), rect.Height() );
		}
		if( number == 1 ) {
			BuddyListSetupView* slist = dynamic_cast<BuddyListSetupView*>( view );		
			if( slist )
				slist->DoResizeStuff( rect.Width(), rect.Height() );
		}
	}
	
	BTab::Select( vw );
}

//-------------------------------------------------------------------------

void PeopleTab::Deselect() {

	// Done adding/removing stuff; commit the changes
	if( number == 1 )
		list->Commit();
	
	BTab::Deselect();
}

//=========================================================================
