#ifndef BUDDY_LIST_H
#define BUDDY_LIST_H

#include <Window.h>
#include <TabView.h>
#include <MenuItem.h>
#include <OutlineListView.h>
#include <TextView.h>
#include <PictureButton.h>
#include <Alert.h>
#include <string.h>
#include "BitmapView.h"
#include "constants.h"
#include "GenList.h"

//-------------------------------------------------------------------------


class ClickableListView : public BOutlineListView
{
	public:
		ClickableListView( BRect, char*, list_view_type, uint32 );
		~ClickableListView();
		virtual void MouseDown( BPoint cursor );
		
	private:
		BPopUpMenu* userPopup;
};


//-------------------------------------------------------------------------


class BuddyListView : public BView
{
	friend class BuddyList;

	public:
		BuddyListView(BRect frame);
		void DoResizeStuff( float width, float height );
		
	protected:
		virtual void Draw( BRect );

	private:
		BScrollView* scroller;
		ClickableListView* online;
};


//-------------------------------------------------------------------------


class BuddyListSetupView : public BView 
{
	public:
		BuddyListSetupView(BRect frame);
		void AddItem( editType );
		void EditItem();
		void RemoveItem();
		void DoResizeStuff( float width, float height );
		
	protected:
		virtual void Draw( BRect );
		virtual void AttachedToWindow();
		
	private:
		bool buttonsDone;
		BScrollView* scroller;
		BOutlineListView* buddies;
		BTextView* buddyEdit;
		BFont baseFont;
		BPictureButton* addBuddyBtn;
		BPictureButton* addGroupBtn;		
		BPictureButton* removeBtn;
};


//-------------------------------------------------------------------------


class BuddyListInfoView : public BView 
{
	friend class BuddyList;

	public:
		BuddyListInfoView(BRect frame);
		virtual void Pulse();

	private:
		void DoIdleStuff();
		unsigned idleCount;
		bool doIdleTime;		
};


//-------------------------------------------------------------------------


class BuddyList : public BWindow 
{
	friend class PeopleTab;

	public:
		BuddyList(BRect frame); 
		~BuddyList();
		
		virtual	bool QuitRequested();
		virtual void MessageReceived( BMessage *message );

	private:

		void LoadPrefs();

		BMenuBar *menubar;
		BuddyListView* onlineview;
		BuddyListSetupView* setupview;
		BitmapView* logoView;
		BuddyListInfoView* iView;
};


//-------------------------------------------------------------------------


class PeopleTab : public BTab {

	public:
		PeopleTab( int32 num, BuddyList* mn );
		void SetControlView( BView* );
		virtual void Select( BView* owner );
		virtual void Deselect();
		
	private:
		BView* view;
		BuddyList* main;
		int32 number;
};


//-------------------------------------------------------------------------


#endif //BUDDY_LIST_H
