#include <Application.h>
#include <Messenger.h>
#include <Message.h>
#include <Roster.h>
#include <Window.h>
#include <View.h>
#include <MenuBar.h>
#include <Menu.h>
#include <Entry.h>
#include <Alert.h>
#include <Path.h>
#include <MenuItem.h>
#include <Box.h>
#include <FilePanel.h>
#include <ScrollView.h>
#include <TextView.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <SupportKit.h>
#include <ctype.h>

#include "Say.h"
#include "Globals.h"
#include "constants.h"
#include "MiscStuff.h"
#include "ChatWindow.h"
#include "PeopleEdit.h"
#include "HTMLStuff.h"
#include "Globals.h"

const rgb_color kBlack = {0,0,0,0};
const rgb_color kThis = {255,0,0,0};		// Red: A statement from THIS aim client
const rgb_color kThat = {0,0,255,0};		// Blue: A statement from the OTHER aim client


ChatWindow::ChatWindow( unsigned int pr, char* iname, char* dname, BRect frame )
			: BWindow(frame, dname, B_TITLED_WINDOW, 0, B_ALL_WORKSPACES )
{
	strcpy( actualName, iname );
	strcpy( displayName, dname );
	proto = pr;
	popupOnReceive = false;
	showFontColorSizes = false;
	hasNewMessage = false;
	prefixNewMessages = false;
	enterIsNewline = false;
	
	lastWidth = lastHeight = -1;
	
	_InitWindow();
	srand(unsigned(time(NULL)));
	
	// Get the screen name for this client
	comm->GetProgramUser( MyScreenName );

	// Tell the main app that the window has been opened
	BMessage* sendMessage = new BMessage( BEAIM_IM_WINDOW_OPENED );
	sendMessage->AddInt32( "wtype", (int32)USER_MESSAGE_TYPE );
	sendMessage->AddString( "userid", actualName );
	sendMessage->AddPointer( "new_window", this );
	sendMessage->AddInt32( "proto", (int32)proto );
	PostAppMessage( sendMessage );
}

//-----------------------------------------------------

// Initialize the window.
void ChatWindow::_InitWindow(void) {

	BRect r;
	BMenu *menu;
	BMenuItem *item;
	float HeightCalc;
	
	// Add the menu bar
	r = Bounds();
	menubar = new BMenuBar(r, "menu_bar");
	AddChild(menubar);
	
	// add the "last message" view
	BRect lastRect = Bounds();
	lastRect.top = menubar->Bounds().bottom + 1.0;
	lastRect.bottom = lastRect.top + 13;
	lastView = new PointerStringView( lastRect, "lastmsg", "(no messages)", B_FOLLOW_TOP | B_FOLLOW_LEFT_RIGHT,
								B_WILL_DRAW | B_FULL_UPDATE_ON_RESIZE );
	lastView->SetViewColor( 125, 125, 125 );
	lastView->SetLowColor( 125, 125, 125 );
	//lastView->SetFont( be_bold_font );
	lastView->SetFontSize(10);
	lastView->SetHighColor( 255, 255, 255 );
	lastView->SetAlignment( B_ALIGN_RIGHT );
	AddChild( lastView );
	
	// Set up the Divider
	divider = 0.8;
	topUIMargin = menubar->Bounds().bottom + lastRect.Height() + 1.0;
	midUIMargin = 9;
	botUIMargin = 37;
	totalUIMargin = topUIMargin + midUIMargin + botUIMargin;

	// Set up the view rectangles
	BRect textframe = Bounds();
	HeightCalc = textframe.bottom - textframe.top - totalUIMargin;
	textframe.right -= B_V_SCROLL_BAR_WIDTH;
	BRect editframe = textframe;		
	textframe.top = topUIMargin;
	textframe.bottom = HeightCalc*divider + topUIMargin;	
	editframe.top = textframe.bottom + midUIMargin;
	editframe.bottom -= botUIMargin;

	// Add the text (conversation) BHTMLView
	BRect textrect = textframe;
	textrect.OffsetTo(B_ORIGIN);
	textrect.InsetBy( 2.0, 2.0 );
	textview = new FancyTextView( textframe, "text_view", textrect, B_FOLLOW_ALL,
								  B_PULSE_NEEDED | B_WILL_DRAW | B_NAVIGABLE_JUMP );
	AddChild(textscrollview = new BScrollView("text_scroll_view", textview,
				B_FOLLOW_LEFT_RIGHT, 0, false, true, B_NO_BORDER));
	textview->SetDoesUndo(true);
	
	// Add the message editing BHTMLView
	BRect editrect = editframe;
	editrect.OffsetTo(B_ORIGIN);
	editrect.InsetBy( 2.0, 2.0 );
	editview = new HTMLView( true, editframe, "edit_view", editrect,
				B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_PULSE_NEEDED | B_NAVIGABLE );
	AddChild(editscrollview = new BScrollView("edit_scroll_view", editview,
				B_FOLLOW_LEFT_RIGHT, 0, false, true, B_NO_BORDER));
	editview->SetDoesUndo(true);
	editview->SetMaxBytes( 1024 );
	editview->ResetFontToBase();
	editview->MakeFocus(true);	

	// Add the divider bar (currently for looks only)
	BRect divframe = Bounds();
	divframe.top = textframe.bottom + 1;
	divframe.bottom = editframe.top - 1;
	divframe.right += 1;
	dragger = new BBox( divframe, "dragger", B_FOLLOW_LEFT_RIGHT, 
					   B_WILL_DRAW | B_FRAME_EVENTS | B_FULL_UPDATE_ON_RESIZE, B_FANCY_BORDER );		
	AddChild( dragger );
	
	// Add the bottom view
	BRect btRect = Bounds();
	btRect.top = btRect.bottom - botUIMargin;
	btView = new BottomView( btRect );
	AddChild( btView );

	// Add File menu to menu bar	
	menu = new BMenu("File");
	menu->AddItem(new BMenuItem("Close", new BMessage(MENU_FILE_CLOSE), 'W'));
	menu->AddSeparatorItem();
	menu->AddItem(saveitem=new BMenuItem("Save as" B_UTF8_ELLIPSIS,
					new BMessage(MENU_FILE_SAVEAS)));
	saveitem->SetEnabled(false);
	menu->AddItem(item=new BMenuItem("Page Setup" B_UTF8_ELLIPSIS,
					new BMessage(MENU_FILE_PAGESETUP)));
	item->SetEnabled(false);
	menu->AddItem(item=new BMenuItem("Print" B_UTF8_ELLIPSIS,
					new BMessage(MENU_FILE_PRINT), 'P'));
	item->SetEnabled(false);
	menu->AddSeparatorItem();
	menu->AddItem(item=new BMenuItem("Quit", new BMessage(B_QUIT_REQUESTED), 'Q'));
	item->SetTarget(APP_SIGNATURE);
	menubar->AddItem(menu);
	
	// Add the Edit menu to the menu bar
	menu = new BMenu("Edit");
	menu->AddItem(miUndo=new BMenuItem("Undo", new BMessage(B_UNDO), 'Z'));
	miUndo->SetTarget(editview);
	menu->AddSeparatorItem();
	menu->AddItem(miCut=new BMenuItem("Cut", new BMessage(B_CUT), 'X'));
	miCut->SetTarget(editview);
	menu->AddItem(miCopy=new BMenuItem("Copy", new BMessage(B_COPY), 'C'));
	miCopy->SetTarget(editview);
	menu->AddItem(miPaste=new BMenuItem("Paste", new BMessage(B_PASTE), 'V'));
	miPaste->SetTarget(editview);
	menu->AddSeparatorItem();
	menu->AddItem(miSelectAll=new BMenuItem("Select All", new BMessage(B_SELECT_ALL), 'A'));
	miSelectAll->SetTarget(editview);
	menubar->AddItem(menu);	
	
	// Add the View menu to the menubar
	menu = new BMenu( "Window" );
	BMenu* workspaceView = new BMenu( "Show this window" );
	workspaceView->SetRadioMode( true );
	BMenuItem* miThisWorkspace = new BMenuItem("Only in This Workspace", new BMessage(CHATWINDOW_SHOW_IN_THIS_WORKSPACE));
	workspaceView->AddItem( miThisWorkspace );
	workspaceView->AddItem( miAllWorkspaces = new BMenuItem("In All Workspaces", new BMessage(CHATWINDOW_SHOW_IN_ALL_WORKSPACES)) );
	menu->AddItem(workspaceView);	
	menu->AddItem(miPopupOnReceive = new BMenuItem("Popup on Message Receive", new BMessage(CHATWINDOW_POPUP_ON_RECEIVE), 'P'));
	menu->AddItem(miShowFontColorSizes = new BMenuItem("Show Font Colors and Sizes", new BMessage(CHATWINDOW_SHOW_FONT_COLORS_SIZES)));
	menu->AddItem(miEnableLinks = new BMenuItem("Enable web/email links", new BMessage(CHATWINDOW_ENABLE_LINKS)));
	menu->AddSeparatorItem();
	menu->AddItem(new BMenuItem("Previous Chat Window", new BMessage(BEAIM_PREV_CHAT_WINDOW), ','));
	menu->AddItem(new BMenuItem("Next Chat Window", new BMessage(BEAIM_NEXT_CHAT_WINDOW), '.'));
	menu->AddItem(item=new BMenuItem("Jump to Buddy List", new BMessage(BEAIM_OPEN_BUDDY_LIST), '/'));
	item->SetTarget(APP_SIGNATURE);
	menubar->AddItem(menu);
	
	// Add the Font menu to the menubar
	menu = new BMenu("Style");
	menu->SetEnabled( false );
	menu->AddItem(miNormal=new BMenuItem("Normal", new BMessage(CHATWINDOW_FONT_NORMAL), 'N'));
	miNormal->SetTarget(editview);
	menu->AddItem(miBold=new BMenuItem("Bold", new BMessage(CHATWINDOW_FONT_BOLD), 'B'));
	miBold->SetTarget(editview);		
	menu->AddItem(miItalic=new BMenuItem("Italic", new BMessage(CHATWINDOW_FONT_ITALIC), 'I'));
	miItalic->SetTarget(editview);
	menu->AddSeparatorItem();
	menu->AddItem(miS1=new BMenuItem("Size 1", new BMessage(CHATWINDOW_FONT_SIZE_1), '1'));
	miS1->SetTarget(editview);
	menu->AddItem(miS2=new BMenuItem("Size 2", new BMessage(CHATWINDOW_FONT_SIZE_2), '2'));
	miS2->SetTarget(editview);
	menu->AddItem(miS3=new BMenuItem("Size 3", new BMessage(CHATWINDOW_FONT_SIZE_3), '3'));
	miS3->SetTarget(editview);
	menu->AddItem(miS4=new BMenuItem("Size 4", new BMessage(CHATWINDOW_FONT_SIZE_4), '4'));
	miS4->SetTarget(editview);
	menu->AddItem(miS5=new BMenuItem("Size 5", new BMessage(CHATWINDOW_FONT_SIZE_5), '5'));
	miS5->SetTarget(editview);
	menu->AddItem(miS6=new BMenuItem("Size 6", new BMessage(CHATWINDOW_FONT_SIZE_6), '6'));
	miS6->SetTarget(editview);
	menu->AddItem(miS7=new BMenuItem("Size 7", new BMessage(CHATWINDOW_FONT_SIZE_7), '7'));
	miS7->SetTarget(editview);			
	menubar->AddItem(menu);		

	// Add the <ScreenName> menu to the menubar
	BMenuItem* buddyItem;
	menu = new BMenu( displayName );
	menu->AddItem(buddyItem = new BMenuItem("Add to Buddy List" B_UTF8_ELLIPSIS,
							  new BMessage(BEAIM_TRY_ADD_PERSON) ));
	menu->AddItem(buddyItem = new BMenuItem("Get Info" B_UTF8_ELLIPSIS,
							  new BMessage(BEAIM_GET_PERSON_INFO) ));
	menu->AddSeparatorItem();
	menu->AddItem(buddyItem = new BMenuItem("Warn", NULL ));
	buddyItem->SetEnabled(false);
	menu->AddItem(buddyItem = new BMenuItem("Block", NULL ));
	buddyItem->SetEnabled(false);
	menubar->AddItem(menu);

	// Set the BTextControl's base font	and attributes
	rgb_color black;
	black.red = black.green = black.blue = 0;
	chatFont.SetSize( 12.0 );
	textview->SetBaseFontAndColor( chatFont, black );
	textview->MakeEditable( false );
	textview->SetStylable( true );
	editview->SetBaseFontAndColor( chatFont, black );
	editview->MakeEditable( true );
	editview->SetStylable( true );
	
	// load the default prefs settings
	if( prefs->ReadBool( "cw_aw", false ) ) {
		SetWorkspaces( B_ALL_WORKSPACES );
		miAllWorkspaces->SetMarked( true );
	} else {
		SetWorkspaces( B_CURRENT_WORKSPACE );
		miThisWorkspace->SetMarked( true );
	}
	if( prefs->ReadBool( "cw_pu", false ) ) {
		popupOnReceive = true;
		miPopupOnReceive->SetMarked( true );
	}
	if( prefs->ReadBool( "cw_fc", true ) ) {
		showFontColorSizes = true;
		textview->SetShowFontColorSizes( true );
		miShowFontColorSizes->SetMarked( true );
	} else
		textview->SetShowFontColorSizes( false );
	if( prefs->ReadBool( "cw_lk", true ) ) {
		linksEnabled = true;
		textview->SetShowLinks( true );
		miEnableLinks->SetMarked( true );
	} else
		textview->SetShowLinks( false );
	prefixNewMessages = prefs->ReadBool( "cw_p>", false );
	enterIsNewline = prefs->ReadBool( "cw_en", false );
}

//-----------------------------------------------------

// Adjust the size of the BTextView's text rectangles
// when the window is resized.
void ChatWindow::FrameResized(float width, float height) {

	BScrollBar* vert;

	// resize the windows themselves
	float newHeight = (height - totalUIMargin) * divider;
	textscrollview->ResizeTo( width, newHeight );
	editscrollview->ResizeTo( width, height - totalUIMargin - newHeight );
	editscrollview->MoveTo( 0, topUIMargin + midUIMargin + newHeight );
	dragger->MoveTo( 0, topUIMargin + newHeight + 1 );
	
	// muck with the scrollview's scrollbars
	vert = textscrollview->ScrollBar(B_VERTICAL);
	if( vert ) {
		vert->SetValue( 0 );
	}

	// cheesy hack alert!	
	if( lastWidth == width && lastHeight == height )
		;//
	lastWidth = width;
	lastHeight = height;
		
	// set the new text rects
	BRect textrect = textview->Bounds();
	textrect.InsetBy( 2.0, 2.0 );
	textview->SetTextRect( textrect );
	textrect = editview->Bounds();
	textrect.InsetBy( 2.0, 2.0 );
	editview->SetTextRect( textrect );
}

//-----------------------------------------------------

void ChatWindow::MessageReceived( BMessage *message ) {

	// Some variables needed for various messages
	BMessenger messenger(APP_SIGNATURE);
	BMessage* sendMessage;
	char* text;

	switch(message->what) {

		// Fire off a BEAIM_SEND_IM message, and clear the editview
		case BEAIM_ATTEMPT_SEND_IM:
			if( editview->IsEmpty() )	// don't send if nothing is there
				return;
			text = editview->GetFormattedMessage();
			sendMessage = new BMessage(BEAIM_SEND_MESSAGE);
			sendMessage->AddString( "send_to", actualName );			
			sendMessage->AddString( "message", text );
			AddMyStatement( editview->GetFormattedMessage() );
			editview->SetText( "" );
			editview->ResetFontToBase();
			PostAppMessage( sendMessage );
			delete text;
			break;
			
		case BEAIM_INCOMING_IM:
			ProcessIncomingIM( message );
			break;
			
		case BEAIM_RELOAD_PREF_SETTINGS:
			LoadPrefs();
			break;			
			
		case CHATWINDOW_SHOW_IN_THIS_WORKSPACE:
			SetWorkspaces( B_CURRENT_WORKSPACE );
			break;
			
		case CHATWINDOW_SHOW_IN_ALL_WORKSPACES:
			SetWorkspaces( B_ALL_WORKSPACES );
			break;
			
		case CHATWINDOW_POPUP_ON_RECEIVE:
			miPopupOnReceive->SetMarked( !miPopupOnReceive->IsMarked() );
			popupOnReceive = miPopupOnReceive->IsMarked();
			break;
			
		case CHATWINDOW_SHOW_FONT_COLORS_SIZES:
			miShowFontColorSizes->SetMarked( !miShowFontColorSizes->IsMarked() );
			showFontColorSizes = miShowFontColorSizes->IsMarked();
			textview->SetShowFontColorSizes( showFontColorSizes );
			break;
			
		case CHATWINDOW_ENABLE_LINKS:
			miEnableLinks->SetMarked( !miEnableLinks->IsMarked() );
			linksEnabled = miEnableLinks->IsMarked();
			textview->SetShowLinks( linksEnabled );
			break;
		
		case B_CANCEL:
		case MENU_FILE_CLOSE:
			QuitRequested();
			Close();
			break;

		case BEAIM_GET_PERSON_INFO:
			sendMessage = new BMessage(BEAIM_OPEN_USER_WINDOW);
			sendMessage->AddInt32( "proto", proto );
			sendMessage->AddString( "userid", actualName );			
			sendMessage->AddInt32( "wtype", (int32)USER_INFO_TYPE );
			sendMessage->AddPointer( "poswindow", this );
			PostAppMessage( sendMessage );
			break;

		case BEAIM_TRY_ADD_PERSON:
			AddPerson(message);
			break;

		case BEAIM_NEXT_CHAT_WINDOW:
		case BEAIM_PREV_CHAT_WINDOW:
			message->AddString( "userid", actualName );
			message->AddInt32( "proto", (int32)proto );		
			message->AddInt32( "wtype", (int32)USER_MESSAGE_TYPE );

		case BEAIM_ADD_PERSON:
			be_app->PostMessage( message );
			break;			

		case CHATWINDOW_FONT_SIZE_1:
		case CHATWINDOW_FONT_SIZE_2:
		case CHATWINDOW_FONT_SIZE_3:
		case CHATWINDOW_FONT_SIZE_4:
		case CHATWINDOW_FONT_SIZE_5:
		case CHATWINDOW_FONT_SIZE_6:
		case CHATWINDOW_FONT_SIZE_7:
		case CHATWINDOW_FONT_NORMAL:
		case CHATWINDOW_FONT_ITALIC:
		case CHATWINDOW_FONT_BOLD:
			SetStyles( message );			
			break;
			
		default:
			BWindow::MessageReceived(message);
			break;
	}
}

//-----------------------------------------------------

// the cancel function
void ChatWindow::DispatchMessage( BMessage* msg, BHandler* handler ) {

	// the enter key does a bunch of stuff, depending on prefs...
	if( msg->what == B_KEY_DOWN )
		if( msg->FindString("bytes")[0] == B_ENTER ) {

			// mod+enter... depends on pref
			if( modifiers() & B_COMMAND_KEY ) {
				if( !enterIsNewline ) {
					editview->Insert("\n");
					editview->ScrollToSelection();
					return;
				}
				// not a newline insert, so make it "un-mod-ed"
				msg->ReplaceInt32( "modifiers", 32 );
			}

			// shift-enter always inserts a line break
			else if( modifiers() & B_SHIFT_KEY ) {
				editview->Insert("\n");
				editview->ScrollToSelection();
				return;
			}

			// plain old enter... depends on pref
			else {
				if( enterIsNewline ) {
					editview->Insert("\n");
					editview->ScrollToSelection();
					return;
				}
			}
	}

	// our work here is done... dispatch normally
	BWindow::DispatchMessage( msg, handler );
}

//-----------------------------------------------------

bool ChatWindow::QuitRequested()
{
	// Tell the main app that the window has been closed
	BMessage* sendMessage = new BMessage( BEAIM_IM_WINDOW_CLOSED );
	sendMessage->AddInt32( "wtype", (int32)USER_MESSAGE_TYPE );
	sendMessage->AddString( "userid", actualName );
	sendMessage->AddInt32( "proto", (int32)proto );
	PostAppMessage( sendMessage );
	
	return(true);
}

//-----------------------------------------------------

void ChatWindow::MenusBeginning() {

	// Adjust the Edit menu... if the read-only TextView is selected, Cut, Paste and Undo
	// should be disabled. Also, the targets on Copy/SelectAll need be correct.
	
	if( textview->IsFocus() ) {
		miSelectAll->SetTarget( textview );
		miCopy->SetTarget( textview );
		miCut->SetEnabled( false );
		miPaste->SetEnabled( false );
		miUndo->SetEnabled( false );				
	}
	
	if( editview->IsFocus() ) {
		miSelectAll->SetTarget( editview );
		miCopy->SetTarget( editview );
		miCut->SetEnabled( true );
		miPaste->SetEnabled( true );
		miUndo->SetEnabled( true );	
	}	
}

//-----------------------------------------------------

void ChatWindow::ProcessIncomingIM( BMessage* incomingIM ) {

	HTMLParser parse;
	styleList styles;
	bool empty = textview->IsEmpty();
	textview->ClearInsertStuff();

	// insert the other person's name (blue font)
	textview->SetFontAttribute( B_BOLD_FACE, true );
	textview->SetFontColor( kThat );
	textview->InsertSomeText( (char*)incomingIM->FindString("userid") );
	textview->InsertSomeText( ": " );
	textview->ResetFontToBase();
	
	// now parse and insert the HTML
	parse.Parse( (char*)incomingIM->FindString("message") );
	styles = parse.Styles();
	textview->AddStyledText( parse.ParsedString(), styles );
	delete styles;

	// Finalize the insertion, and update the timestamp
	textview->AddStatement();
	UpdateLastMessage();

	// Play the sound
	if( empty )
		sounds->PlaySound( WS_NEWMSG );
	else
		sounds->PlaySound( WS_MSGRECEIVE );

	// popup the window if the user has specified that option
	if( popupOnReceive )
		Popup();
	
	// update the window title if prefixing with '√' is on
	hasNewMessage = true;
	if( prefixNewMessages ) {
		char newTitle[DISPLAY_NAME_MAX+10];
		sprintf( newTitle, "√ %s", displayName );
		SetTitle( newTitle );
	}
}

//-----------------------------------------------------

void ChatWindow::AddMyStatement( char* statement ) {

	HTMLParser parse;
	styleList styles;
	textview->ClearInsertStuff();

	// insert our name (red font)
	textview->SetFontAttribute( B_BOLD_FACE, true );
	textview->SetFontColor( kThis );
	textview->InsertSomeText( MyScreenName );
	textview->InsertSomeText( ": " );
	textview->ResetFontToBase();
	
	// now parse and insert the HTML
	parse.Parse( statement );
	styles = parse.Styles();
	textview->AddStyledText( parse.ParsedString(), styles );
	delete styles;

	// Finalize the insertion, and update the timestamp
	textview->AddStatement();
	UpdateLastMessage();
	
	// if that was a new message we just responded to,
	//  then take out the '√' from the titlebar
	if( hasNewMessage && prefixNewMessages )
		SetTitle( displayName );
	hasNewMessage = false;
}

//-----------------------------------------------------

void ChatWindow::Popup() {
	
	// Check to see if this window is already displayed in all workspaces
	// if not, move it to the current workspace		
	if( Workspaces() != B_ALL_WORKSPACES )
		SetWorkspaces( B_CURRENT_WORKSPACE );

	// if the window is hidden, pop it up
	if( IsHidden() || !IsFront() ) {
		Show();
		Activate();
	}
}

//-----------------------------------------------------

void ChatWindow::LoadPrefs() {

	// put in or take out the '√', depending on what the pref setting is
	prefixNewMessages = prefs->ReadBool( "cw_p>", false );
	if( !prefixNewMessages )
		SetTitle( displayName );
	else if( hasNewMessage ) {
		char newTitle[DISPLAY_NAME_MAX+10];
		sprintf( newTitle, "√ %s", displayName );
		SetTitle( newTitle );	
	}
	
	// enable/disable enter inserts a newline
	enterIsNewline = prefs->ReadBool( "cw_en", false );
}

//-----------------------------------------------------

void ChatWindow::SetStyles( BMessage* message ) {
	
	int32 start, finish, attrib;
	text_run_array* curStyles = NULL;
	BFont font, font2;
	uint16 curMask;
	
	// Get the current selection, and the current styles for that selection
	editview->GetSelection( &start, &finish );
	curStyles = editview->RunArray( start, finish );
	font = curStyles->runs[0].font;
	curMask = font.Face();
		
	bool on;
		
	switch( message->what ) {
		
		// set the font back to the normal font	
		case CHATWINDOW_FONT_NORMAL:
			font = editview->GetBaseFont();
			editview->SetFontAndColor( start, finish, &font );
			break;
		
		// make it italic
		case CHATWINDOW_FONT_ITALIC:
			if( message->what == CHATWINDOW_FONT_ITALIC )
				attrib = B_ITALIC_FACE;

			editview->GetFontAndColor( start, &font2 );

			on = !(curMask & attrib);
			
			on = true;

				for( int i = 0; i < curStyles->count; ++i ) {
					curMask = curStyles->runs[i].font.Face();
					if( on ) curMask = ((curMask & B_REGULAR_FACE) ? 0 : curMask) | attrib;
					else curMask = (curMask == attrib) ? B_REGULAR_FACE : curMask & (~attrib);
					curStyles->runs[i].font.SetFace( curMask );
				}
				editview->SetRunArray( start, finish, curStyles );
				break;
		
		// make it bold
		case CHATWINDOW_FONT_BOLD:
			Say( "bold" );
			break;
		
		// font size change
		default:
			font.SetSize( ConvertFontSize(message->what - CHATWINDOW_FONT_BASE) );
			editview->SetFontAndColor( start, finish, &font );
			break;
	}
		
	delete curStyles;
}

//-----------------------------------------------------

void ChatWindow::AddPerson( BMessage* msg ) {

	BRect frame( 0, 0, 200, 85 );
	PersonAddEditWindow* dlg;

	// are there any groups?
	if( !list->CountGroups() ) {
		Say( "You need to add a group first." );
		return;
	}

	// tweak the frame and open the dialog
	windows->MakeDialogFrame( frame, this );
 	dlg = new PersonAddEditWindow( frame, this, ET_ADDPERSON, NULL );
 	dlg->SetInitialName( displayName );
 	dlg->SetNow( true );
 	dlg->Show();
}

//-----------------------------------------------------

void ChatWindow::UpdateLastMessage() {
	
	char theTime[100];
	
	int rnd = (int)(random() % 1979);
	if( rnd == 42 )
		lastView->SetText( "Run! The squirrels are coming!" );
	else {
		time_t now;
		time(&now);		
		strftime( theTime, 100, "%b %e, %Y (%I:%M %p)", localtime(&now) );
		lastView->SetText( theTime );
	}
}

//=====================================================

BottomView::BottomView( BRect frame )
		   : BView( frame, "bottom_view", B_FOLLOW_BOTTOM | B_FOLLOW_LEFT_RIGHT,
		   								  B_WILL_DRAW | B_NAVIGABLE_JUMP )
{
	SetViewColor( 216, 216, 216 );
	
	BRect btnFrame = Bounds();
	btnFrame.right -= 7;
	btnFrame.left = btnFrame.right - 60;
	btnFrame.top = 7;
	btnFrame.bottom = 27;
	sendButton = new BButton( btnFrame, "send_button", "Send", new BMessage(BEAIM_ATTEMPT_SEND_IM),
										B_FOLLOW_RIGHT | B_FOLLOW_TOP, B_NAVIGABLE | B_WILL_DRAW );
	sendButton->MakeDefault( true );
	
	AddChild( sendButton );
}

//=====================================================

PointerStringView::PointerStringView( BRect frame, const char *name, const char *text, 
					   uint32 resizingMode = B_FOLLOW_LEFT | B_FOLLOW_TOP, 
					   uint32 flags = B_WILL_DRAW )
				 : BStringView( frame, name, text, resizingMode, flags )
{
}

//-----------------------------------------------------

void PointerStringView::MouseMoved( BPoint where, uint32 code, const BMessage *msg ) {

	be_app->SetCursor(B_HAND_CURSOR);
	BStringView::MouseMoved(where, code, msg);
}

//=====================================================