#include <Bitmap.h>
#include <Alert.h>
#include <Resources.h>
#include <SupportKit.h>
#include <Application.h>
#include <ctype.h>
#include <stdlib.h>
#include "constants.h"
#include "MiscStuff.h"
#include "BitmapView.h"
#include "Say.h"

//-------------------------------------------------------------------------

void GetBitmapFromResources( BBitmap*& themap, int32 resid, char *resfile ) 
{
	BResources res; 
   	status_t err; 
   	
   	size_t  reslength;
   	const char * resbuf;
   	BMessage archive;

	// 	Open resource file
	BFile file( resfile, B_READ_ONLY );  

	// if the file loads OK, that is
   	if ( (file.InitCheck() || (err = res.SetTo(&file)) ) == B_NO_ERROR )  {

		// Find and load resource
   		resbuf = (char*)res.FindResource( B_RAW_TYPE, resid, &reslength );
   							
   		if( resbuf ) {
   		
			// Inflate and unarchive BBitmap
	   		archive.Unflatten( resbuf );	
   			themap = new BBitmap( &archive );	

		} else {
	   		(new BAlert("", "Error while reading resource from file.", "OK"))->Go();
		}
		    		
   	} else {	// Error
   		(new BAlert("", "Error opening resource file.", "OK"))->Go();
   		exit(1);
   	}
}

//-------------------------------------------------------------------------

void MakeButtonPicture( BPicture*& pic, BView* view, int32 resid, char *resfile ) 
{
	BBitmap* bitmap = NULL;
	GetBitmapFromResources( bitmap, resid, resfile );
	
	//tempview for creating the picture
	BView *tempView = new BView( BRect(0,0,50,50), "temp", B_FOLLOW_NONE, B_WILL_DRAW );
	view->AddChild(tempView);	
	
	// draw the bitmap into the BPicture
	tempView->BeginPicture(new BPicture); 
	tempView->DrawBitmap(bitmap);
	pic = tempView->EndPicture();

	//get rid of tempview
	view->RemoveChild(tempView);
	delete tempView;	
	delete bitmap;
}

//-------------------------------------------------------------------------

int ConvertFontSize( int fontSize ) {

	switch( fontSize ) {
		case 1: return 8; 	break;
		case 2: return 10; 	break;	
		case 3: return 12; 	break;
		case 4: return 14; 	break;
		case 5: return 18; 	break;
		case 6: return 24; 	break;
		case 7: return 38; 	break;					
		default: return 12; break;
	}	
}

//-------------------------------------------------------------------------

void PostAppMessage( BMessage* msg ) {
	be_app->PostMessage( msg );
	delete msg;
}

//-------------------------------------------------------------------------

void GetAppPath() {

	app_info winfo;
	be_app->GetAppInfo( &winfo );
}

//-------------------------------------------------------------------------

bool CompareScreenNames( char* a, char* b ) {

	while( *a && *a == ' ' ) ++a;
	while( *b && *b == ' ' ) ++b;
	if( !(*a) && !(*b) )
		return true;		
	if( !(*a) || !(*b) )
		return false;

	while( *a && *b ) {
		while( *a && *a == ' ' ) ++a;
		while( *b && *b == ' ' ) ++b;
		if( !(*a) && !(*b) )
			return true;
		if( !(*a) || !(*b) )
			return false;
		if( tolower(*a) != tolower(*b) )
			return false;
		++a;
		++b;
	}
	return true;
}

//-------------------------------------------------------------------------

void MakeFilenameFromUsername( char* fullpath, char* username ) {

	char* uname2 = new char[strlen(username)+1];
	strcpy( uname2, username );
	
	char* a = uname2;
	char* b = a;
	int endoff = 0;
	
	while( *a ) {
		if( *a != ' ' ) {
			if( b != a ) {
				endoff += int(a-b);
				b = a;
			}
			++b;
		}
		*(a-endoff) = tolower(*a);	
		++a;
	}
	if( b != a )
		endoff += int(a-b);
	*(a-endoff) = '\0';
	
	sprintf( fullpath, "/boot/home/config/settings/BeAIM/user-%s", uname2 );
	delete uname2;
}

//-------------------------------------------------------------------------