#include <Application.h>
#include <Bitmap.h>
#include <Alert.h>
#include <MenuItem.h>
#include <stdlib.h>
#include "Globals.h"
#include "PeopleEdit.h"
#include "constants.h"
#include "Say.h"


PersonAddEditWindow::PersonAddEditWindow( BRect frame, BWindow* mk, editType mode, PersonInfo* pi )
	 				: BWindow(frame, "", B_TITLED_WINDOW, B_NOT_RESIZABLE | B_NOT_ZOOMABLE )
{
	info = pi;
	maker = mk;
	now = false;
	
	// if a NULL PersonInfo was passed in, make one
	if( !info ) {
		info = new PersonInfo;
		info->setupitem = NULL;
		info->item = NULL;
		info->parent = NULL;
		info->name[0] = '\0';
		info->status = BS_OFFLINE;
		info->proto = AIM_PROTOCOL;
	}
		
	// Set the correct title based on the edit mode
	switch( mode ) {
		case ET_ADDPERSON:
			ResizeBy( 0, 23 );		
			SetTitle( "Add Person" );
			msgID = BEAIM_ADD_PERSON;
			break;
		case ET_ADDGROUP:
			SetTitle( "Add Group" );
			msgID = BEAIM_ADD_GROUP;
			break;
		case ET_PERSONEDIT:
			ResizeBy( 0, 23 );		
			SetTitle( "Edit Person" );
			msgID = BEAIM_CHANGE_PERSON;
			break;
		case ET_GROUPEDIT:
			SetTitle( "Edit Group" );
			msgID = BEAIM_CHANGE_GROUP;
			break;
	}
	
	// Set up the view
	BRect aRect( Bounds() );
	personView = new PersonAddEditView( aRect, msgID, 
					 mode == ET_PERSONEDIT && info->parent ? info->parent->group : NULL );
	AddChild( personView );

	// Copy in the text, if needed
	if( mode == ET_PERSONEDIT || mode == ET_GROUPEDIT )
		personView->personName->SetText( pi->name );

	// if you want the text to be selected when the dialog starts, move this line 
	//   up before the text is copied in
	personView->personName->MakeFocus( true );
}


PersonAddEditWindow::~PersonAddEditWindow() {

}


bool PersonAddEditWindow::QuitRequested()
{
	return(true);
}


void PersonAddEditWindow::MessageReceived(BMessage* message)
{
	switch(message->what)
	{
		case B_OK:
			Save();
						
		case B_CANCEL:
			Close();
			break;
	
		default:
			BWindow::MessageReceived(message);
	}
}


void PersonAddEditWindow::Save() {

	// make a message
	BMessage* sendMessage = new BMessage( msgID );
	
	// set the appropriate fields	
	if( msgID == BEAIM_CHANGE_PERSON || msgID == BEAIM_CHANGE_GROUP )
		sendMessage->AddString( "oldname", info->name );
	sendMessage->AddString( "newname", personView->personName->Text() );
	sendMessage->AddInt32( "oldproto", (unsigned int)info->proto );
	sendMessage->AddInt32( "newproto", (unsigned int)info->proto );
	if( msgID == BEAIM_ADD_PERSON || msgID == BEAIM_CHANGE_PERSON )
		sendMessage->AddString( "group", (char*)personView->groupMenu->FindMarked()->Label() );
	if( msgID == BEAIM_ADD_PERSON ) {
		sendMessage->AddBool( "commitnow", now );
	}

	// send it (dontcha just love unnecessary comments?)
	maker->PostMessage( sendMessage );

	// now kill that bad boy	
	delete sendMessage;
}


void PersonAddEditWindow::SetInitialName( char* name ) {
	personView->personName->SetText(name);
}

// the cancel function
void PersonAddEditWindow::DispatchMessage( BMessage* msg, BHandler* handler ) {

	// if it's a cancel key, post a B_CANCEL message
	if( msg->what == B_KEY_DOWN )
		if( msg->FindString("bytes")[0] == B_ESCAPE ) {
			PostMessage( new BMessage(B_CANCEL) );
			return;
		}
	
	// our work here is done... dispatch normally
	BWindow::DispatchMessage( msg, handler );
}

//-----------------------------------------------------------------------------

PersonAddEditView::PersonAddEditView( BRect rect, uint32 id, const char* gp )
	   	   : BView(rect, "person_addedit_view", B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW)
{
	SetViewColor( 216, 216, 216 );
	msgID = id;
	BRect btnRect(62,50,122,70);
	BMenuItem* groupItem;
	menuField = NULL;

	// Construct the views
	personName = new BTextControl( BRect(8,15,192,35), "user_id", 
					(id==BEAIM_ADD_GROUP || id==BEAIM_CHANGE_GROUP) ? "Group:" : "User ID:", "", NULL );
	personName->SetViewColor( 216, 216, 216 );
	personName->SetDivider( 50 );
	//personName->TextView()->SetMaxBytes( SCREEN_NAME_MAX );
	personName->MakeFocus( true );

	// Make the group selector (if needed)
	if( msgID == BEAIM_ADD_PERSON || msgID == BEAIM_CHANGE_PERSON ) {

		char group[DISPLAY_NAME_MAX];	

		// Get the first group, and create the menu w/ it
		list->GetGroup( 0, group );
		groupMenu = new BMenu( group );
		groupMenu->SetRadioMode( true );
		groupMenu->SetLabelFromMarked( true );
		
		// Add all the rest of the groups
		for( unsigned int i = 0; i < list->CountGroups(); ++i ) {
			list->GetGroup( i, group );
			groupItem = new BMenuItem(group, new BMessage(ADDEDIT_SELECTION_CHANGED));
			if( gp )
				groupItem->SetMarked( (bool)(strcasecmp(group,gp) == 0) );
			else
				groupItem->SetMarked( (bool)(i == 0) );
			groupMenu->AddItem( groupItem );
		}
			
		// Now create the menufield
		BRect menuRect( 8, 37, 192, 62 );
		menuField = new BMenuField( menuRect, "group_select", "Group:", groupMenu );
		menuField->SetDivider( 53 );
		btnRect.OffsetBy( 0, 23 );
		ResizeBy( 0, 23 );
	}

	// Make the Save and Cancel buttons
	SaveButton = new BButton( btnRect, "savebutton", "Save", new BMessage((unsigned long)B_OK) );
	SaveButton->MakeDefault( true );
	btnRect.OffsetBy( 70, 0 );
	CancelButton = new BButton( btnRect, "cancelbutton", "Cancel", new BMessage(B_CANCEL) );
	
	// Now add them to the window
	AddChild( personName );
	if( menuField )
		AddChild( menuField );	
	AddChild( SaveButton );
	AddChild( CancelButton );
	//AddChild( Divider );
	
	personName->MakeFocus();
}

void PersonAddEditView::KeyDown( const char *bytes, int32 numBytes ) {

}
