#ifndef _TCP_NET_THREAD_
#define _TCP_NET_THREAD_

#include <Looper.h>
#include <kernel/OS.h>
#include <net/socket.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <errno.h>
#include "constants.h"
#include "NetThread.h"

// Connection status
const unsigned long s_inval = 0;
const unsigned long s_open = 1;
const unsigned long s_bound = 2;
const unsigned long s_connected = 4;

// waitfor return status
const int WF_GOTDATA = 1;
const int WF_CONTINUE = 2;
const int WF_CONNDROPPED = 4;
const int WF_ERROR = 8;

const uint32 BEAIM_NET_DATA_AVAILABLE = 'dAvB';
const uint32 BEAIM_NET_CONNECTED = 'cnTD';
const uint32 BEAIM_NET_LOST_CONNECTION = 'lScn';

// errors... ugh
const int TCP_INTERNALERROR = 1;
const int TCP_COULDNOTCONNECT = 2;
const int TCP_COULDNOTRESOLVEHOSTNAME = 3;
const int TCP_UNKNOWNERROR = 4;

// The net thread base class... actually a BLooper
class TCPNetThread : public NetThreadBase {

	public:
		TCPNetThread( char* );

		// Generalized virtual functions
		virtual void Close();	
		virtual void Login( BMessage* );
		virtual void Logout( BMessage* );
		
	protected:

		// Stuff for the threads
		bool MakeReadThread();
		void WakeReadThread();
		void KillReadThread();
		bool MakeConnectThread();
		void KillConnectThread();		

		// Generalized virtual functions
		virtual bool IsConnected()  {return true;};//{ return !(net == NULL); };
		virtual bool Connect( char*, int32, bool=true );
		virtual void Disconnect();
		virtual void MessageReceived( BMessage* );

		// Handler functions
		virtual void Connected();
		virtual void Disconnected();
		virtual void DataWaiting();
		virtual	void HandleError( int error, const char* message = 0 );

		// socket functions
		void socket_reset();
		void socket_close();
		void socket_connect( char*, int );
		ssize_t socket_send( const void *buf, size_t size );
		ssize_t socket_recv( void *buf, size_t size );
		int socket_waitfor_data();
			
	private:

		// internal notification
		void nowConnected();
	
		// thread functions
		static int32 socket_read_entry( void *arg );
		int32 do_socket_read();
		static int32 connect_entry( void *arg );
		int32 do_connect();
		
		// socket/connection data
		int m_socket;
		unsigned long m_state;
		char conHost[512];
		int32 conPort;		
		
		// should there be a read thread & event calls?
		bool fullServiceConnection;

		// thread declarations
		thread_id read_thread, connect_thread;
		
		//  Connection status
		static unsigned long s_open;
		static unsigned long s_bound;
		static unsigned long s_connected;
};

#endif