#include <stdio.h>
#include <PopUpMenu.h>
#include <MenuItem.h>
#include <SupportKit.h>
#include <Bitmap.h>
#include "TrayIcon.h"
#include "Say.h"
#include "MiscStuff.h"
#include "constants.h"

//---------------------------------------------------------

const char* DESKBAR_SIGNATURE = "application/x-vnd.Be-TSKB";

//---------------------------------------------------------

TrayIcon::TrayIcon(BRect frame, BBitmap* active, BBitmap* inactive)
	: BView(frame, "AIMDeskbar", B_FOLLOW_NONE, B_WILL_DRAW | B_WILL_ACCEPT_FIRST_CLICK)
{
	ActiveIcon = active;
	InactiveIcon = inactive;
}


//---------------------------------------------------------

TrayIcon::TrayIcon( BMessage *archive )
	: BView( archive )
{
	BMessage			bits;
	archive->FindMessage("ActiveIcon", &bits);
	ActiveIcon = new BBitmap(&bits);
	
	bits.MakeEmpty();
	archive->FindMessage("InactiveIcon", &bits);
	InactiveIcon = new BBitmap(&bits);
}

//---------------------------------------------------------

TrayIcon::~TrayIcon()
{
	if(ActiveIcon) delete ActiveIcon;
	if(InactiveIcon) delete InactiveIcon;
}

//---------------------------------------------------------

void TrayIcon::AddTrayIcon()
{	
	BMessage message, reply;
	
	// set the command constant 
	message = new BMessage(B_GET_PROPERTY); 
	
	message.AddSpecifier("Replicant", "AIMDeskbar");	// 5. B_NAME_SPECIFIER
	message.AddSpecifier("Shelf");						// 4. B_DIRECT_SPECIFIER
	message.AddSpecifier("View", (int32) 0);			// 3. B_INDEX_SPECIFIER
	message.AddSpecifier("View", (int32) 0);			// 2. B_INDEX_SPECIFIER
	message.AddSpecifier("Window", "Deskbar");			// 1. B_NAME_SPECIFIER
	
	// send the message and fetch the result (with large timeout)
	BMessenger(DESKBAR_SIGNATURE).SendMessage(&message, &reply, 1000000, 1000000);
	
	if(reply.what == B_MESSAGE_NOT_UNDERSTOOD) {
	
	  	// Install the deskbar replicant
		BMessage archiveMsg(B_ARCHIVED_OBJECT);
		
		BBitmap*	active;
		BBitmap*	inactive;
		GetBitmapFromResources(active, 962);
		GetBitmapFromResources(inactive, 962);
		
		// Archive a view into the msg.
		TrayIcon* replicant = new TrayIcon(BRect(0, 0, 15, 15), active, inactive);
		replicant->Archive(&archiveMsg);
		
		// Send the object to the Tracker Deskbar application
		BMessenger messenger(DESKBAR_SIGNATURE, -1, NULL);
		messenger.SendMessage(&archiveMsg);
	}
}

//---------------------------------------------------------

void TrayIcon::RemoveTrayIcon()
{
	BMessage message, reply; 

	// set the command constant 
	message = new BMessage(B_DELETE_PROPERTY); 
	
	// add specifiers
	message.AddSpecifier("Replicant", "AIMDeskbar");	// 5. B_NAME_SPECIFIER
	message.AddSpecifier("Shelf");						// 4. B_DIRECT_SPECIFIER
	message.AddSpecifier("View", (int32) 0);			// 3. B_INDEX_SPECIFIER
	message.AddSpecifier("View", (int32) 0);			// 2. B_INDEX_SPECIFIER
	message.AddSpecifier("Window", "Deskbar");			// 1. B_NAME_SPECIFIER
	
	// send the message and fetch the result (with large timeout)
	BMessenger(DESKBAR_SIGNATURE).SendMessage(&message, &reply,1000000,1000000); 
}

//---------------------------------------------------------

void TrayIcon::AttachedToWindow() 
{ 
	// set view color based on parent
	if (Parent()) 
		SetViewColor(Parent()->ViewColor()); 

	// call inherited class
	BView::AttachedToWindow();
}

//---------------------------------------------------------

void TrayIcon::Draw(BRect updateRect)
{
	BView::Draw(updateRect);
	SetDrawingMode(B_OP_OVER);

	// just draw the active icon for now until we can impliment status
	if(ActiveIcon) DrawBitmap(ActiveIcon);
}

//---------------------------------------------------------

void TrayIcon::MouseDown(BPoint where)
{
	uint32 mouseButtons;
	Window()->CurrentMessage()->FindInt32("buttons", (long*)&mouseButtons);
	switch (mouseButtons)
	{
		// left: send a message to BeAIM telling it to toggle minimization
		case B_PRIMARY_MOUSE_BUTTON: {
			BMessage message(BEAIM_TOGGLE_HIDDEN);
			BMessenger(APP_SIGNATURE).SendMessage(&message);
			break;
		}

		// right: popup a menu (which does very little at the moment...)
		case B_SECONDARY_MOUSE_BUTTON: {
			BPopUpMenu* popup = new BPopUpMenu("Status",false,false);
			BMenuItem*  item;
			item = new BMenuItem("Available", 0);
			item->SetEnabled(false);
			popup->AddItem(item);
			item = new BMenuItem("Away", 0);
			item->SetEnabled(false);
			popup->AddItem(item);
			item = new BMenuItem("Gone fishin'", 0);
			item->SetEnabled(false);
			popup->AddItem(item);
			popup->AddSeparatorItem();
			popup->AddItem(new BMenuItem("Quit BeAIM", new BMessage(B_QUIT_REQUESTED)));
			popup->SetTargetForItems(BMessenger(APP_SIGNATURE, -1, NULL));
			
			ConvertToScreen(&where); 
			popup->SetTargetForItems(BMessenger(APP_SIGNATURE, -1, NULL));
			popup->Go( where,true,false,true );
		}
	}
}	

//---------------------------------------------------------

void TrayIcon::MessageReceived(BMessage* msg)
{
	switch(msg->what) {
	
		case B_SET_PROPERTY:
			//inMessage->FindInt32("status",(int32 *) &my_status);
			//Invalidate(Bounds());
			break;

		default:
			BView::MessageReceived(msg);
			break;
	}
}	

//---------------------------------------------------------

TrayIcon* TrayIcon::Instantiate(BMessage* archive)
{
	if(!validate_instantiation(archive, "TrayIcon"))
		return NULL;
		
	return new TrayIcon(archive);
}

//---------------------------------------------------------

status_t TrayIcon::Archive( BMessage *archive, bool deep ) const
{
	// archive this bad boy
	BView::Archive(archive, deep);
	
	// save the icons
	BMessage	bits(B_ARCHIVED_OBJECT);
	if(ActiveIcon) {
		ActiveIcon->Archive(&bits);
		archive->AddMessage("ActiveIcon", &bits);
		bits.MakeEmpty();
	}
	
	if(InactiveIcon) {
		InactiveIcon->Archive(&bits);
		archive->AddMessage("InactiveIcon", &bits);
	}
	
	archive->AddString("add_on", APP_SIGNATURE);
	return B_OK;
}

//---------------------------------------------------------
